/* header file for TBLCF.DLL */

#define WRITE_BLOCK_CHECK

typedef enum {	/* type of BDM target */
	CF_BDM=0,
	JTAG=1
} target_type_e;

typedef enum {	/* type of reset action required */
	BDM_MODE=0,
	NORMAL_MODE=1
} target_mode_e;

typedef enum {	/* target reset detection state */
	RESET_NOT_DETECTED=0,
	RESET_DETECTED=1
} reset_detection_e;

typedef enum {	/* target reset state */
	RSTO_ACTIVE=0,
	RSTO_INACTIVE=1
} reset_state_e;

typedef struct { /* state of BDM communication */
	reset_state_e reset_state;
	reset_detection_e reset_detection;
} bdmcf_status_t;

unsigned char WINAPI tblcf_dll_version(void); 				/* returns DLL version in BCD */
unsigned char WINAPI tblcf_init(void);						/* initialises USB and returns number of devices found */
unsigned char WINAPI tblcf_open(unsigned char device_no); 	/* opens a device with given number (0...), returns 0 on success and 1 on error */
void WINAPI tblcf_close(void);								/* closes currently open device */
unsigned int WINAPI tblcf_get_version(void);				/* returns hardware & software version of the cable in BCD format - SW version in lower byte and HW version in upper byte */
unsigned char WINAPI tblcf_get_last_sts(void); 				/* returns status of the last command: 0 on sucess and non-zero on failure */
unsigned char WINAPI tblcf_request_boot(void); 				/* requests bootloader execution on new power-up, returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_set_target_type(target_type_e target_type); 	/* sets target MCU type; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_target_reset(target_mode_e target_mode); 	/* resets the target to normal or BDM mode; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_bdm_sts(bdmcf_status_t *bdmcf_status); 		/* fills user supplied structure with current state of the BDM communication channel; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_target_halt(void);				/* brings the target into BDM mode; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_target_go(void); 				/* starts target execution from current PC address; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_target_step(void);				/* steps over a single target instruction; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_resynchronize(void); 			/* resynchronizes communication with the target (in case of noise, etc.); returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_assert_ta(unsigned char duration_10us); 		/* asserts the TA signal for the specified time (in 10us ticks); returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_read_creg(unsigned int address, unsigned long int * result);		/* reads control register at the specified address and writes its contents into the supplied buffer; returns 0 on success and non-zero on failure */
void WINAPI tblcf_write_creg(unsigned int address, unsigned long int value);				/* writes control register at the specified address; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_read_dreg(unsigned char dreg_index, unsigned long int * result); /* reads the specified debug register and writes its contents into the supplied buffer; returns 0 on success and non-zero on failure */
void WINAPI tblcf_write_dreg(unsigned char dreg_index, unsigned long int value); 			/* writes specified debug register */
unsigned char WINAPI tblcf_read_reg(unsigned char reg_index, unsigned long int * result);	/* reads the specified register and writes its contents into the supplied buffer; returns 0 on success and non-zero on failure */
void WINAPI tblcf_write_reg(unsigned char reg_index, unsigned long int value); 				/* writes specified register */
unsigned char WINAPI tblcf_read_mem8(unsigned long int address, unsigned char * result); 	/* reads byte from the specified address; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_read_mem16(unsigned long int address, unsigned int * result);	/* reads word from the specified address; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_read_mem32(unsigned long int address, unsigned long int * result); /* reads long word from the specified address; returns 0 on success and non-zero on failure */
void WINAPI tblcf_write_mem8(unsigned long int address, unsigned char value);				/* writes byte at the specified address */
void WINAPI tblcf_write_mem16(unsigned long int address, unsigned int value); 				/* writes word at the specified address */
void WINAPI tblcf_write_mem32(unsigned long int address, unsigned long int value);			/* writes long word at the specified address */
unsigned char WINAPI tblcf_read_block8(unsigned long int address, unsigned long int bytecount, unsigned char *buffer);		/* reads the requested number of bytes from target memory from the supplied address and stores results into the user supplied buffer; uses byte accesses only; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_read_block16(unsigned long int address, unsigned long int bytecount, unsigned char *buffer); 	/* reads the requested number of bytes from target memory from the supplied address and stores results into the user supplied buffer; uses word accesses; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_read_block32(unsigned long int address, unsigned long int bytecount, unsigned char *buffer);		/* reads the requested number of bytes from target memory from the supplied address and stores results into the user supplied buffer; uses long word accesses; returns 0 on success and non-zero on failure */
unsigned char WINAPI tblcf_write_block8(unsigned long int address, unsigned long int bytecount, unsigned char *buffer); 	/* writes the requested number of bytes to target memory from the supplied address; uses byte accesses only; returns 0 on success and non-zero on failure (must be compiled with WRITE_BLOCK_CHECK, otherwise always returns 0) */
unsigned char WINAPI tblcf_write_block16(unsigned long int address, unsigned long int bytecount, unsigned char *buffer); 	/* writes the requested number of bytes to target memory at the supplied address; uses word accesses; returns 0 on success and non-zero on failure (must be compiled with WRITE_BLOCK_CHECK, otherwise always returns 0) */
unsigned char WINAPI tblcf_write_block32(unsigned long int address, unsigned long int bytecount, unsigned char *buffer); 	/* writes the requested number of bytes to target memory at the supplied address; uses long word accesses; returns 0 on success and non-zero on failure (must be compiled with WRITE_BLOCK_CHECK, otherwise always returns 0) */
unsigned char WINAPI tblcf_jtag_sel_shift(unsigned char mode);										/* JTAG - go from RUN-TEST/IDLE to SHIFT-DR (mode==0) or SHIFT-IR (mode!=0) state */
unsigned char WINAPI tblcf_jtag_sel_reset(void);													/* JTAG - go from RUN-TEST/IDLE to TEST-LOGIC-RESET state */
void WINAPI tblcf_jtag_write(unsigned char bit_count, unsigned char exit, unsigned char *buffer);	/* JTAG - write data; parameter exit: ==0 : stay in SHIFT-xx, !=0 : go to RUN-TEST/IDLE when done; data: shifted in LSB (last byte) first, unused bits (if any) are in the MSB (first) byte */
unsigned char WINAPI tblcf_jtag_read(unsigned char bit_count, unsigned char exit, unsigned char *buffer);	/* JTAG - read data; parameter exit: ==0 : stay in SHIFT-xx, !=0 : go to RUN-TEST/IDLE when done; data: shifted in LSB (last byte) first, unused bits (if any) are in the MSB (first) byte */
